      SUBROUTINE RG(NM,N,A,WR,WI,MATZ,Z,IV1,FV1,IERR) 
C 
      INTEGER N,NM,IS1,IS2,IERR,MATZ 
      DOUBLE PRECISION A(NM,N),WR(N),WI(N),Z(NM,N),FV1(N) 
      INTEGER IV1(N) 
C 
C     THIS SUBROUTINE CALLS THE RECOMMENDED SEQUENCE OF 
C     SUBROUTINES FROM THE EIGENSYSTEM SUBROUTINE PACKAGE (EISPACK) 
C     TO FIND THE EIGENVALUES AND EIGENVECTORS (IF DESIRED) 
C     OF A REAL GENERAL MATRIX. 
C 
C     ON INPUT 
C 
C        NM  MUST BE SET TO THE ROW DIMENSION OF THE TWO-DIMENSIONAL 
C        ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C        DIMENSION STATEMENT. 
C 
C        N  IS THE ORDER OF THE MATRIX  A. 
C 
C        A  CONTAINS THE REAL GENERAL MATRIX. 
C 
C        MATZ  IS AN INTEGER VARIABLE SET EQUAL TO ZERO IF 
C        ONLY EIGENVALUES ARE DESIRED.  OTHERWISE IT IS SET TO 
C        ANY NON-ZERO INTEGER FOR BOTH EIGENVALUES AND EIGENVECTORS. 
C 
C     ON OUTPUT 
C 
C        WR  AND  WI  CONTAIN THE REAL AND IMAGINARY PARTS, 
C        RESPECTIVELY, OF THE EIGENVALUES.  COMPLEX CONJUGATE 
C        PAIRS OF EIGENVALUES APPEAR CONSECUTIVELY WITH THE 
C        EIGENVALUE HAVING THE POSITIVE IMAGINARY PART FIRST. 
C 
C        Z  CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGENVECTORS 
C        IF MATZ IS NOT ZERO.  IF THE J-TH EIGENVALUE IS REAL, THE 
C        J-TH COLUMN OF  Z  CONTAINS ITS EIGENVECTOR.  IF THE J-TH 
C        EIGENVALUE IS COMPLEX WITH POSITIVE IMAGINARY PART, THE 
C        J-TH AND (J+1)-TH COLUMNS OF  Z  CONTAIN THE REAL AND 
C        IMAGINARY PARTS OF ITS EIGENVECTOR.  THE CONJUGATE OF THIS 
C        VECTOR IS THE EIGENVECTOR FOR THE CONJUGATE EIGENVALUE. 
C 
C        IERR  IS AN INTEGER OUTPUT VARIABLE SET EQUAL TO AN ERROR 
C           COMPLETION CODE DESCRIBED IN THE DOCUMENTATION FOR HQR 
C           AND HQR2.  THE NORMAL COMPLETION CODE IS ZERO. 
C 
C        IV1  AND  FV1  ARE TEMPORARY STORAGE ARRAYS. 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
      IF (N .LE. NM) GO TO 10 
      IERR = 10 * N 
      GO TO 50 
C 
   10 CALL  BALANC(NM,N,A,IS1,IS2,FV1) 
      CALL  ELMHES(NM,N,IS1,IS2,A,IV1) 
      IF (MATZ .NE. 0) GO TO 20 
C     .......... FIND EIGENVALUES ONLY .......... 
      CALL  HQR(NM,N,IS1,IS2,A,WR,WI,IERR) 
      GO TO 50 
C     .......... FIND BOTH EIGENVALUES AND EIGENVECTORS .......... 
   20 CALL  ELTRAN(NM,N,IS1,IS2,A,IV1,Z) 
      CALL  HQR2(NM,N,IS1,IS2,A,WR,WI,Z,IERR) 
      IF (IERR .NE. 0) GO TO 50 
      CALL  BALBAK(NM,N,IS1,IS2,FV1,N,Z) 
   50 RETURN 
      END 
      SUBROUTINE BALANC(NM,N,A,LOW,IGH,SCALE) 
C 
      INTEGER I,J,K,L,M,N,JJ,NM,IGH,LOW,IEXC 
      DOUBLE PRECISION A(NM,N),SCALE(N) 
      DOUBLE PRECISION C,F,G,R,S,B2,RADIX 
      LOGICAL NOCONV 
C 
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALANCE, 
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH. 
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971). 
C 
C     THIS SUBROUTINE BALANCES A REAL MATRIX AND ISOLATES 
C     EIGENVALUES WHENEVER POSSIBLE. 
C 
C     ON INPUT 
C 
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C          DIMENSION STATEMENT. 
C 
C        N IS THE ORDER OF THE MATRIX. 
C 
C        A CONTAINS THE INPUT MATRIX TO BE BALANCED. 
C 
C     ON OUTPUT 
C 
C        A CONTAINS THE BALANCED MATRIX. 
C 
C        LOW AND IGH ARE TWO INTEGERS SUCH THAT A(I,J) 
C          IS EQUAL TO ZERO IF 
C           (1) I IS GREATER THAN J AND 
C           (2) J=1,...,LOW-1 OR I=IGH+1,...,N. 
C 
C        SCALE CONTAINS INFORMATION DETERMINING THE 
C           PERMUTATIONS AND SCALING FACTORS USED. 
C 
C     SUPPOSE THAT THE PRINCIPAL SUBMATRIX IN ROWS LOW THROUGH IGH 
C     HAS BEEN BALANCED, THAT P(J) DENOTES THE INDEX INTERCHANGED 
C     WITH J DURING THE PERMUTATION STEP, AND THAT THE ELEMENTS 
C     OF THE DIAGONAL MATRIX USED ARE DENOTED BY D(I,J).  THEN 
C        SCALE(J) = P(J),    FOR J = 1,...,LOW-1 
C                 = D(J,J),      J = LOW,...,IGH 
C                 = P(J)         J = IGH+1,...,N. 
C     THE ORDER IN WHICH THE INTERCHANGES ARE MADE IS N TO IGH+1, 
C     THEN 1 TO LOW-1. 
C 
C     NOTE THAT 1 IS RETURNED FOR IGH IF IGH IS ZERO FORMALLY. 
C 
C     THE ALGOL PROCEDURE EXC CONTAINED IN BALANCE APPEARS IN 
C     BALANC  IN LINE.  (NOTE THAT THE ALGOL ROLES OF IDENTIFIERS 
C     K,L HAVE BEEN REVERSED.) 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
      RADIX = 16.0D0 
C 
      B2 = RADIX * RADIX 
      K = 1 
      L = N 
      GO TO 100 
C     .......... IN-LINE PROCEDURE FOR ROW AND 
C                COLUMN EXCHANGE .......... 
   20 SCALE(M) = J 
      IF (J .EQ. M) GO TO 50 
C 
      DO 30 I = 1, L 
         F = A(I,J) 
         A(I,J) = A(I,M) 
         A(I,M) = F 
   30 CONTINUE 
C 
      DO 40 I = K, N 
         F = A(J,I) 
         A(J,I) = A(M,I) 
         A(M,I) = F 
   40 CONTINUE 
C 
   50 GO TO (80,130), IEXC 
C     .......... SEARCH FOR ROWS ISOLATING AN EIGENVALUE 
C                AND PUSH THEM DOWN .......... 
   80 IF (L .EQ. 1) GO TO 280 
      L = L - 1 
C     .......... FOR J=L STEP -1 UNTIL 1 DO -- .......... 
  100 DO 120 JJ = 1, L 
         J = L + 1 - JJ 
C 
         DO 110 I = 1, L 
            IF (I .EQ. J) GO TO 110 
            IF (A(J,I) .NE. 0.0D0) GO TO 120 
  110    CONTINUE 
C 
         M = L 
         IEXC = 1 
         GO TO 20 
  120 CONTINUE 
C 
      GO TO 140 
C     .......... SEARCH FOR COLUMNS ISOLATING AN EIGENVALUE 
C                AND PUSH THEM LEFT .......... 
  130 K = K + 1 
C 
  140 DO 170 J = K, L 
C 
         DO 150 I = K, L 
            IF (I .EQ. J) GO TO 150 
            IF (A(I,J) .NE. 0.0D0) GO TO 170 
  150    CONTINUE 
C 
         M = K 
         IEXC = 2 
         GO TO 20 
  170 CONTINUE 
C     .......... NOW BALANCE THE SUBMATRIX IN ROWS K TO L .......... 
      DO 180 I = K, L 
  180 SCALE(I) = 1.0D0 
C     .......... ITERATIVE LOOP FOR NORM REDUCTION .......... 
  190 NOCONV = .FALSE. 
C 
      DO 270 I = K, L 
         C = 0.0D0 
         R = 0.0D0 
C 
         DO 200 J = K, L 
            IF (J .EQ. I) GO TO 200 
            C = C + DABS(A(J,I)) 
            R = R + DABS(A(I,J)) 
  200    CONTINUE 
C     .......... GUARD AGAINST ZERO C OR R DUE TO UNDERFLOW .......... 
         IF (C .EQ. 0.0D0 .OR. R .EQ. 0.0D0) GO TO 270 
         G = R / RADIX 
         F = 1.0D0 
         S = C + R 
  210    IF (C .GE. G) GO TO 220 
         F = F * RADIX 
         C = C * B2 
         GO TO 210 
  220    G = R * RADIX 
  230    IF (C .LT. G) GO TO 240 
         F = F / RADIX 
         C = C / B2 
         GO TO 230 
C     .......... NOW BALANCE .......... 
  240    IF ((C + R) / F .GE. 0.95D0 * S) GO TO 270 
         G = 1.0D0 / F 
         SCALE(I) = SCALE(I) * F 
         NOCONV = .TRUE. 
C 
         DO 250 J = K, N 
  250    A(I,J) = A(I,J) * G 
C 
         DO 260 J = 1, L 
  260    A(J,I) = A(J,I) * F 
C 
  270 CONTINUE 
C 
      IF (NOCONV) GO TO 190 
C 
  280 LOW = K 
      IGH = L 
      RETURN 
      END 
      SUBROUTINE ELMHES(NM,N,LOW,IGH,A,INT) 
C 
      INTEGER I,J,M,N,LA,NM,IGH,KP1,LOW,MM1,MP1 
      DOUBLE PRECISION A(NM,N) 
      DOUBLE PRECISION X,Y 
      INTEGER INT(IGH) 
C 
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ELMHES, 
C     NUM. MATH. 12, 349-368(1968) BY MARTIN AND WILKINSON. 
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 339-358(1971). 
C 
C     GIVEN A REAL GENERAL MATRIX, THIS SUBROUTINE 
C     REDUCES A SUBMATRIX SITUATED IN ROWS AND COLUMNS 
C     LOW THROUGH IGH TO UPPER HESSENBERG FORM BY 
C     STABILIZED ELEMENTARY SIMILARITY TRANSFORMATIONS. 
C 
C     ON INPUT 
C 
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C          DIMENSION STATEMENT. 
C 
C        N IS THE ORDER OF THE MATRIX. 
C 
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING 
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED, 
C          SET LOW=1, IGH=N. 
C 
C        A CONTAINS THE INPUT MATRIX. 
C 
C     ON OUTPUT 
C 
C        A CONTAINS THE HESSENBERG MATRIX.  THE MULTIPLIERS 
C          WHICH WERE USED IN THE REDUCTION ARE STORED IN THE 
C          REMAINING TRIANGLE UNDER THE HESSENBERG MATRIX. 
C 
C        INT CONTAINS INFORMATION ON THE ROWS AND COLUMNS 
C          INTERCHANGED IN THE REDUCTION. 
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED. 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
      LA = IGH - 1 
      KP1 = LOW + 1 
      IF (LA .LT. KP1) GO TO 200 
C 
      DO 180 M = KP1, LA 
         MM1 = M - 1 
         X = 0.0D0 
         I = M 
C 
         DO 100 J = M, IGH 
            IF (DABS(A(J,MM1)) .LE. DABS(X)) GO TO 100 
            X = A(J,MM1) 
            I = J 
  100    CONTINUE 
C 
         INT(M) = I 
         IF (I .EQ. M) GO TO 130 
C     .......... INTERCHANGE ROWS AND COLUMNS OF A .......... 
         DO 110 J = MM1, N 
            Y = A(I,J) 
            A(I,J) = A(M,J) 
            A(M,J) = Y 
  110    CONTINUE 
C 
         DO 120 J = 1, IGH 
            Y = A(J,I) 
            A(J,I) = A(J,M) 
            A(J,M) = Y 
  120    CONTINUE 
C     .......... END INTERCHANGE .......... 
  130    IF (X .EQ. 0.0D0) GO TO 180 
         MP1 = M + 1 
C 
         DO 160 I = MP1, IGH 
            Y = A(I,MM1) 
            IF (Y .EQ. 0.0D0) GO TO 160 
            Y = Y / X 
            A(I,MM1) = Y 
C 
            DO 140 J = M, N 
  140       A(I,J) = A(I,J) - Y * A(M,J) 
C 
            DO 150 J = 1, IGH 
  150       A(J,M) = A(J,M) + Y * A(J,I) 
C 
  160    CONTINUE 
C 
  180 CONTINUE 
C 
  200 RETURN 
      END 
      SUBROUTINE HQR(NM,N,LOW,IGH,H,WR,WI,IERR) 
C 
      INTEGER I,J,K,L,M,N,EN,LL,MM,NA,NM,IGH,ITN,ITS,LOW,MP2,ENM2,IERR 
      DOUBLE PRECISION H(NM,N),WR(N),WI(N) 
      DOUBLE PRECISION P,Q,R,S,T,W,X,Y,ZZ,NORM,TST1,TST2 
      LOGICAL NOTLAS 
C 
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE HQR, 
C     NUM. MATH. 14, 219-231(1970) BY MARTIN, PETERS, AND WILKINSON. 
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 359-371(1971). 
C 
C     THIS SUBROUTINE FINDS THE EIGENVALUES OF A REAL 
C     UPPER HESSENBERG MATRIX BY THE QR METHOD. 
C 
C     ON INPUT 
C 
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C          DIMENSION STATEMENT. 
C 
C        N IS THE ORDER OF THE MATRIX. 
C 
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING 
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED, 
C          SET LOW=1, IGH=N. 
C 
C        H CONTAINS THE UPPER HESSENBERG MATRIX.  INFORMATION ABOUT 
C          THE TRANSFORMATIONS USED IN THE REDUCTION TO HESSENBERG 
C          FORM BY  ELMHES  OR  ORTHES, IF PERFORMED, IS STORED 
C          IN THE REMAINING TRIANGLE UNDER THE HESSENBERG MATRIX. 
C 
C     ON OUTPUT 
C 
C        H HAS BEEN DESTROYED.  THEREFORE, IT MUST BE SAVED 
C          BEFORE CALLING  HQR  IF SUBSEQUENT CALCULATION AND 
C          BACK TRANSFORMATION OF EIGENVECTORS IS TO BE PERFORMED. 
C 
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS, 
C          RESPECTIVELY, OF THE EIGENVALUES.  THE EIGENVALUES 
C          ARE UNORDERED EXCEPT THAT COMPLEX CONJUGATE PAIRS 
C          OF VALUES APPEAR CONSECUTIVELY WITH THE EIGENVALUE 
C          HAVING THE POSITIVE IMAGINARY PART FIRST.  IF AN 
C          ERROR EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT 
C          FOR INDICES IERR+1,...,N. 
C 
C        IERR IS SET TO 
C          ZERO       FOR NORMAL RETURN, 
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED 
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT. 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
      IERR = 0 
      NORM = 0.0D0 
      K = 1 
C     .......... STORE ROOTS ISOLATED BY BALANC 
C                AND COMPUTE MATRIX NORM .......... 
      DO 50 I = 1, N 
C 
         DO 40 J = K, N 
   40    NORM = NORM + DABS(H(I,J)) 
C 
         K = I 
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 50 
         WR(I) = H(I,I) 
         WI(I) = 0.0D0 
   50 CONTINUE 
C 
      EN = IGH 
      T = 0.0D0 
      ITN = 30*N 
C     .......... SEARCH FOR NEXT EIGENVALUES .......... 
   60 IF (EN .LT. LOW) GO TO 1001 
      ITS = 0 
      NA = EN - 1 
      ENM2 = NA - 1 
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT 
C                FOR L=EN STEP -1 UNTIL LOW DO -- .......... 
   70 DO 80 LL = LOW, EN 
         L = EN + LOW - LL 
         IF (L .EQ. LOW) GO TO 100 
         S = DABS(H(L-1,L-1)) + DABS(H(L,L)) 
         IF (S .EQ. 0.0D0) S = NORM 
         TST1 = S 
         TST2 = TST1 + DABS(H(L,L-1)) 
         IF (TST2 .EQ. TST1) GO TO 100 
   80 CONTINUE 
C     .......... FORM SHIFT .......... 
  100 X = H(EN,EN) 
      IF (L .EQ. EN) GO TO 270 
      Y = H(NA,NA) 
      W = H(EN,NA) * H(NA,EN) 
      IF (L .EQ. NA) GO TO 280 
      IF (ITN .EQ. 0) GO TO 1000 
      IF (ITS .NE. 10 .AND. ITS .NE. 20) GO TO 130 
C     .......... FORM EXCEPTIONAL SHIFT .......... 
      T = T + X 
C 
      DO 120 I = LOW, EN 
  120 H(I,I) = H(I,I) - X 
C 
      S = DABS(H(EN,NA)) + DABS(H(NA,ENM2)) 
      X = 0.75D0 * S 
      Y = X 
      W = -0.4375D0 * S * S 
  130 ITS = ITS + 1 
      ITN = ITN - 1 
C     .......... LOOK FOR TWO CONSECUTIVE SMALL 
C                SUB-DIAGONAL ELEMENTS. 
C                FOR M=EN-2 STEP -1 UNTIL L DO -- .......... 
      DO 140 MM = L, ENM2 
         M = ENM2 + L - MM 
         ZZ = H(M,M) 
         R = X - ZZ 
         S = Y - ZZ 
         P = (R * S - W) / H(M+1,M) + H(M,M+1) 
         Q = H(M+1,M+1) - ZZ - R - S 
         R = H(M+2,M+1) 
         S = DABS(P) + DABS(Q) + DABS(R) 
         P = P / S 
         Q = Q / S 
         R = R / S 
         IF (M .EQ. L) GO TO 150 
         TST1 = DABS(P)*(DABS(H(M-1,M-1)) + DABS(ZZ) + DABS(H(M+1,M+1))) 
         TST2 = TST1 + DABS(H(M,M-1))*(DABS(Q) + DABS(R)) 
         IF (TST2 .EQ. TST1) GO TO 150 
  140 CONTINUE 
C 
  150 MP2 = M + 2 
C 
      DO 160 I = MP2, EN 
         H(I,I-2) = 0.0D0 
         IF (I .EQ. MP2) GO TO 160 
         H(I,I-3) = 0.0D0 
  160 CONTINUE 
C     .......... DOUBLE QR STEP INVOLVING ROWS L TO EN AND 
C                COLUMNS M TO EN .......... 
      DO 260 K = M, NA 
         NOTLAS = K .NE. NA 
         IF (K .EQ. M) GO TO 170 
         P = H(K,K-1) 
         Q = H(K+1,K-1) 
         R = 0.0D0 
         IF (NOTLAS) R = H(K+2,K-1) 
         X = DABS(P) + DABS(Q) + DABS(R) 
         IF (X .EQ. 0.0D0) GO TO 260 
         P = P / X 
         Q = Q / X 
         R = R / X 
  170    S = DSIGN(DSQRT(P*P+Q*Q+R*R),P) 
         IF (K .EQ. M) GO TO 180 
         H(K,K-1) = -S * X 
         GO TO 190 
  180    IF (L .NE. M) H(K,K-1) = -H(K,K-1) 
  190    P = P + S 
         X = P / S 
         Y = Q / S 
         ZZ = R / S 
         Q = Q / P 
         R = R / P 
         IF (NOTLAS) GO TO 225 
C     .......... ROW MODIFICATION .......... 
         DO 200 J = K, N 
            P = H(K,J) + Q * H(K+1,J) 
            H(K,J) = H(K,J) - P * X 
            H(K+1,J) = H(K+1,J) - P * Y 
  200    CONTINUE 
C 
         J = MIN0(EN,K+3) 
C     .......... COLUMN MODIFICATION .......... 
         DO 210 I = 1, J 
            P = X * H(I,K) + Y * H(I,K+1) 
            H(I,K) = H(I,K) - P 
            H(I,K+1) = H(I,K+1) - P * Q 
  210    CONTINUE 
         GO TO 255 
  225    CONTINUE 
C     .......... ROW MODIFICATION .......... 
         DO 230 J = K, N 
            P = H(K,J) + Q * H(K+1,J) + R * H(K+2,J) 
            H(K,J) = H(K,J) - P * X 
            H(K+1,J) = H(K+1,J) - P * Y 
            H(K+2,J) = H(K+2,J) - P * ZZ 
  230    CONTINUE 
C 
         J = MIN0(EN,K+3) 
C     .......... COLUMN MODIFICATION .......... 
         DO 240 I = 1, J 
            P = X * H(I,K) + Y * H(I,K+1) + ZZ * H(I,K+2) 
            H(I,K) = H(I,K) - P 
            H(I,K+1) = H(I,K+1) - P * Q 
            H(I,K+2) = H(I,K+2) - P * R 
  240    CONTINUE 
  255    CONTINUE 
C 
  260 CONTINUE 
C 
      GO TO 70 
C     .......... ONE ROOT FOUND .......... 
  270 WR(EN) = X + T 
      WI(EN) = 0.0D0 
      EN = NA 
      GO TO 60 
C     .......... TWO ROOTS FOUND .......... 
  280 P = (Y - X) / 2.0D0 
      Q = P * P + W 
      ZZ = DSQRT(DABS(Q)) 
      X = X + T 
      IF (Q .LT. 0.0D0) GO TO 320 
C     .......... REAL PAIR .......... 
      ZZ = P + DSIGN(ZZ,P) 
      WR(NA) = X + ZZ 
      WR(EN) = WR(NA) 
      IF (ZZ .NE. 0.0D0) WR(EN) = X - W / ZZ 
      WI(NA) = 0.0D0 
      WI(EN) = 0.0D0 
      GO TO 330 
C     .......... COMPLEX PAIR .......... 
  320 WR(NA) = X + P 
      WR(EN) = X + P 
      WI(NA) = ZZ 
      WI(EN) = -ZZ 
  330 EN = ENM2 
      GO TO 60 
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT 
C                CONVERGED AFTER 30*N ITERATIONS .......... 
 1000 IERR = EN 
 1001 RETURN 
      END 
      SUBROUTINE ELTRAN(NM,N,LOW,IGH,A,INT,Z) 
C 
      INTEGER I,J,N,KL,MM,MP,NM,IGH,LOW,MP1 
      DOUBLE PRECISION A(NM,IGH),Z(NM,N) 
      INTEGER INT(IGH) 
C 
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ELMTRANS, 
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON. 
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971). 
C 
C     THIS SUBROUTINE ACCUMULATES THE STABILIZED ELEMENTARY 
C     SIMILARITY TRANSFORMATIONS USED IN THE REDUCTION OF A 
C     REAL GENERAL MATRIX TO UPPER HESSENBERG FORM BY  ELMHES. 
C 
C     ON INPUT 
C 
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C          DIMENSION STATEMENT. 
C 
C        N IS THE ORDER OF THE MATRIX. 
C 
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING 
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED, 
C          SET LOW=1, IGH=N. 
C 
C        A CONTAINS THE MULTIPLIERS WHICH WERE USED IN THE 
C          REDUCTION BY  ELMHES  IN ITS LOWER TRIANGLE 
C          BELOW THE SUBDIAGONAL. 
C 
C        INT CONTAINS INFORMATION ON THE ROWS AND COLUMNS 
C          INTERCHANGED IN THE REDUCTION BY  ELMHES. 
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED. 
C 
C     ON OUTPUT 
C 
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE 
C          REDUCTION BY  ELMHES. 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
C     .......... INITIALIZE Z TO IDENTITY MATRIX .......... 
      DO 80 J = 1, N 
C 
         DO 60 I = 1, N 
   60    Z(I,J) = 0.0D0 
C 
         Z(J,J) = 1.0D0 
   80 CONTINUE 
C 
      KL = IGH - LOW - 1 
      IF (KL .LT. 1) GO TO 200 
C     .......... FOR MP=IGH-1 STEP -1 UNTIL LOW+1 DO -- .......... 
      DO 140 MM = 1, KL 
         MP = IGH - MM 
         MP1 = MP + 1 
C 
         DO 100 I = MP1, IGH 
  100    Z(I,MP) = A(I,MP-1) 
C 
         I = INT(MP) 
         IF (I .EQ. MP) GO TO 140 
C 
         DO 130 J = MP, IGH 
            Z(MP,J) = Z(I,J) 
            Z(I,J) = 0.0D0 
  130    CONTINUE 
C 
         Z(I,MP) = 1.0D0 
  140 CONTINUE 
C 
  200 RETURN 
      END 
      SUBROUTINE HQR2(NM,N,LOW,IGH,H,WR,WI,Z,IERR) 
C 
      INTEGER I,J,K,L,M,N,EN,II,JJ,LL,MM,NA,NM,NN, 
     X        IGH,ITN,ITS,LOW,MP2,ENM2,IERR 
      DOUBLE PRECISION H(NM,N),WR(N),WI(N),Z(NM,N) 
      DOUBLE PRECISION P,Q,R,S,T,W,X,Y,RA,SA,VI,VR,ZZ,NORM,TST1,TST2 
      LOGICAL NOTLAS 
C 
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE HQR2, 
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON. 
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971). 
C 
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS 
C     OF A REAL UPPER HESSENBERG MATRIX BY THE QR METHOD.  THE 
C     EIGENVECTORS OF A REAL GENERAL MATRIX CAN ALSO BE FOUND 
C     IF  ELMHES  AND  ELTRAN  OR  ORTHES  AND  ORTRAN  HAVE 
C     BEEN USED TO REDUCE THIS GENERAL MATRIX TO HESSENBERG FORM 
C     AND TO ACCUMULATE THE SIMILARITY TRANSFORMATIONS. 
C 
C     ON INPUT 
C 
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C          DIMENSION STATEMENT. 
C 
C        N IS THE ORDER OF THE MATRIX. 
C 
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING 
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED, 
C          SET LOW=1, IGH=N. 
C 
C        H CONTAINS THE UPPER HESSENBERG MATRIX. 
C 
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED BY  ELTRAN 
C          AFTER THE REDUCTION BY  ELMHES, OR BY  ORTRAN  AFTER THE 
C          REDUCTION BY  ORTHES, IF PERFORMED.  IF THE EIGENVECTORS 
C          OF THE HESSENBERG MATRIX ARE DESIRED, Z MUST CONTAIN THE 
C          IDENTITY MATRIX. 
C 
C     ON OUTPUT 
C 
C        H HAS BEEN DESTROYED. 
C 
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS, 
C          RESPECTIVELY, OF THE EIGENVALUES.  THE EIGENVALUES 
C          ARE UNORDERED EXCEPT THAT COMPLEX CONJUGATE PAIRS 
C          OF VALUES APPEAR CONSECUTIVELY WITH THE EIGENVALUE 
C          HAVING THE POSITIVE IMAGINARY PART FIRST.  IF AN 
C          ERROR EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT 
C          FOR INDICES IERR+1,...,N. 
C 
C        Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGENVECTORS. 
C          IF THE I-TH EIGENVALUE IS REAL, THE I-TH COLUMN OF Z 
C          CONTAINS ITS EIGENVECTOR.  IF THE I-TH EIGENVALUE IS COMPLEX 
C          WITH POSITIVE IMAGINARY PART, THE I-TH AND (I+1)-TH 
C          COLUMNS OF Z CONTAIN THE REAL AND IMAGINARY PARTS OF ITS 
C          EIGENVECTOR.  THE EIGENVECTORS ARE UNNORMALIZED.  IF AN 
C          ERROR EXIT IS MADE, NONE OF THE EIGENVECTORS HAS BEEN FOUND. 
C 
C        IERR IS SET TO 
C          ZERO       FOR NORMAL RETURN, 
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED 
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT. 
C 
C     CALLS CDIV FOR COMPLEX DIVISION. 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
      IERR = 0 
      NORM = 0.0D0 
      K = 1 
C     .......... STORE ROOTS ISOLATED BY BALANC 
C                AND COMPUTE MATRIX NORM .......... 
      DO 50 I = 1, N 
C 
         DO 40 J = K, N 
   40    NORM = NORM + DABS(H(I,J)) 
C 
         K = I 
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 50 
         WR(I) = H(I,I) 
         WI(I) = 0.0D0 
   50 CONTINUE 
C 
      EN = IGH 
      T = 0.0D0 
      ITN = 30*N 
C     .......... SEARCH FOR NEXT EIGENVALUES .......... 
   60 IF (EN .LT. LOW) GO TO 340 
      ITS = 0 
      NA = EN - 1 
      ENM2 = NA - 1 
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT 
C                FOR L=EN STEP -1 UNTIL LOW DO -- .......... 
   70 DO 80 LL = LOW, EN 
         L = EN + LOW - LL 
         IF (L .EQ. LOW) GO TO 100 
         S = DABS(H(L-1,L-1)) + DABS(H(L,L)) 
         IF (S .EQ. 0.0D0) S = NORM 
         TST1 = S 
         TST2 = TST1 + DABS(H(L,L-1)) 
         IF (TST2 .EQ. TST1) GO TO 100 
   80 CONTINUE 
C     .......... FORM SHIFT .......... 
  100 X = H(EN,EN) 
      IF (L .EQ. EN) GO TO 270 
      Y = H(NA,NA) 
      W = H(EN,NA) * H(NA,EN) 
      IF (L .EQ. NA) GO TO 280 
      IF (ITN .EQ. 0) GO TO 1000 
      IF (ITS .NE. 10 .AND. ITS .NE. 20) GO TO 130 
C     .......... FORM EXCEPTIONAL SHIFT .......... 
      T = T + X 
C 
      DO 120 I = LOW, EN 
  120 H(I,I) = H(I,I) - X 
C 
      S = DABS(H(EN,NA)) + DABS(H(NA,ENM2)) 
      X = 0.75D0 * S 
      Y = X 
      W = -0.4375D0 * S * S 
  130 ITS = ITS + 1 
      ITN = ITN - 1 
C     .......... LOOK FOR TWO CONSECUTIVE SMALL 
C                SUB-DIAGONAL ELEMENTS. 
C                FOR M=EN-2 STEP -1 UNTIL L DO -- .......... 
      DO 140 MM = L, ENM2 
         M = ENM2 + L - MM 
         ZZ = H(M,M) 
         R = X - ZZ 
         S = Y - ZZ 
         P = (R * S - W) / H(M+1,M) + H(M,M+1) 
         Q = H(M+1,M+1) - ZZ - R - S 
         R = H(M+2,M+1) 
         S = DABS(P) + DABS(Q) + DABS(R) 
         P = P / S 
         Q = Q / S 
         R = R / S 
         IF (M .EQ. L) GO TO 150 
         TST1 = DABS(P)*(DABS(H(M-1,M-1)) + DABS(ZZ) + DABS(H(M+1,M+1))) 
         TST2 = TST1 + DABS(H(M,M-1))*(DABS(Q) + DABS(R)) 
         IF (TST2 .EQ. TST1) GO TO 150 
  140 CONTINUE 
C 
  150 MP2 = M + 2 
C 
      DO 160 I = MP2, EN 
         H(I,I-2) = 0.0D0 
         IF (I .EQ. MP2) GO TO 160 
         H(I,I-3) = 0.0D0 
  160 CONTINUE 
C     .......... DOUBLE QR STEP INVOLVING ROWS L TO EN AND 
C                COLUMNS M TO EN .......... 
      DO 260 K = M, NA 
         NOTLAS = K .NE. NA 
         IF (K .EQ. M) GO TO 170 
         P = H(K,K-1) 
         Q = H(K+1,K-1) 
         R = 0.0D0 
         IF (NOTLAS) R = H(K+2,K-1) 
         X = DABS(P) + DABS(Q) + DABS(R) 
         IF (X .EQ. 0.0D0) GO TO 260 
         P = P / X 
         Q = Q / X 
         R = R / X 
  170    S = DSIGN(DSQRT(P*P+Q*Q+R*R),P) 
         IF (K .EQ. M) GO TO 180 
         H(K,K-1) = -S * X 
         GO TO 190 
  180    IF (L .NE. M) H(K,K-1) = -H(K,K-1) 
  190    P = P + S 
         X = P / S 
         Y = Q / S 
         ZZ = R / S 
         Q = Q / P 
         R = R / P 
         IF (NOTLAS) GO TO 225 
C     .......... ROW MODIFICATION .......... 
         DO 200 J = K, N 
            P = H(K,J) + Q * H(K+1,J) 
            H(K,J) = H(K,J) - P * X 
            H(K+1,J) = H(K+1,J) - P * Y 
  200    CONTINUE 
C 
         J = MIN0(EN,K+3) 
C     .......... COLUMN MODIFICATION .......... 
         DO 210 I = 1, J 
            P = X * H(I,K) + Y * H(I,K+1) 
            H(I,K) = H(I,K) - P 
            H(I,K+1) = H(I,K+1) - P * Q 
  210    CONTINUE 
C     .......... ACCUMULATE TRANSFORMATIONS .......... 
         DO 220 I = LOW, IGH 
            P = X * Z(I,K) + Y * Z(I,K+1) 
            Z(I,K) = Z(I,K) - P 
            Z(I,K+1) = Z(I,K+1) - P * Q 
  220    CONTINUE 
         GO TO 255 
  225    CONTINUE 
C     .......... ROW MODIFICATION .......... 
         DO 230 J = K, N 
            P = H(K,J) + Q * H(K+1,J) + R * H(K+2,J) 
            H(K,J) = H(K,J) - P * X 
            H(K+1,J) = H(K+1,J) - P * Y 
            H(K+2,J) = H(K+2,J) - P * ZZ 
  230    CONTINUE 
C 
         J = MIN0(EN,K+3) 
C     .......... COLUMN MODIFICATION .......... 
         DO 240 I = 1, J 
            P = X * H(I,K) + Y * H(I,K+1) + ZZ * H(I,K+2) 
            H(I,K) = H(I,K) - P 
            H(I,K+1) = H(I,K+1) - P * Q 
            H(I,K+2) = H(I,K+2) - P * R 
  240    CONTINUE 
C     .......... ACCUMULATE TRANSFORMATIONS .......... 
         DO 250 I = LOW, IGH 
            P = X * Z(I,K) + Y * Z(I,K+1) + ZZ * Z(I,K+2) 
            Z(I,K) = Z(I,K) - P 
            Z(I,K+1) = Z(I,K+1) - P * Q 
            Z(I,K+2) = Z(I,K+2) - P * R 
  250    CONTINUE 
  255    CONTINUE 
C 
  260 CONTINUE 
C 
      GO TO 70 
C     .......... ONE ROOT FOUND .......... 
  270 H(EN,EN) = X + T 
      WR(EN) = H(EN,EN) 
      WI(EN) = 0.0D0 
      EN = NA 
      GO TO 60 
C     .......... TWO ROOTS FOUND .......... 
  280 P = (Y - X) / 2.0D0 
      Q = P * P + W 
      ZZ = DSQRT(DABS(Q)) 
      H(EN,EN) = X + T 
      X = H(EN,EN) 
      H(NA,NA) = Y + T 
      IF (Q .LT. 0.0D0) GO TO 320 
C     .......... REAL PAIR .......... 
      ZZ = P + DSIGN(ZZ,P) 
      WR(NA) = X + ZZ 
      WR(EN) = WR(NA) 
      IF (ZZ .NE. 0.0D0) WR(EN) = X - W / ZZ 
      WI(NA) = 0.0D0 
      WI(EN) = 0.0D0 
      X = H(EN,NA) 
      S = DABS(X) + DABS(ZZ) 
      P = X / S 
      Q = ZZ / S 
      R = DSQRT(P*P+Q*Q) 
      P = P / R 
      Q = Q / R 
C     .......... ROW MODIFICATION .......... 
      DO 290 J = NA, N 
         ZZ = H(NA,J) 
         H(NA,J) = Q * ZZ + P * H(EN,J) 
         H(EN,J) = Q * H(EN,J) - P * ZZ 
  290 CONTINUE 
C     .......... COLUMN MODIFICATION .......... 
      DO 300 I = 1, EN 
         ZZ = H(I,NA) 
         H(I,NA) = Q * ZZ + P * H(I,EN) 
         H(I,EN) = Q * H(I,EN) - P * ZZ 
  300 CONTINUE 
C     .......... ACCUMULATE TRANSFORMATIONS .......... 
      DO 310 I = LOW, IGH 
         ZZ = Z(I,NA) 
         Z(I,NA) = Q * ZZ + P * Z(I,EN) 
         Z(I,EN) = Q * Z(I,EN) - P * ZZ 
  310 CONTINUE 
C 
      GO TO 330 
C     .......... COMPLEX PAIR .......... 
  320 WR(NA) = X + P 
      WR(EN) = X + P 
      WI(NA) = ZZ 
      WI(EN) = -ZZ 
  330 EN = ENM2 
      GO TO 60 
C     .......... ALL ROOTS FOUND.  BACKSUBSTITUTE TO FIND 
C                VECTORS OF UPPER TRIANGULAR FORM .......... 
  340 IF (NORM .EQ. 0.0D0) GO TO 1001 
C     .......... FOR EN=N STEP -1 UNTIL 1 DO -- .......... 
      DO 800 NN = 1, N 
         EN = N + 1 - NN 
         P = WR(EN) 
         Q = WI(EN) 
         NA = EN - 1 
         IF (Q) 710, 600, 800 
C     .......... REAL VECTOR .......... 
  600    M = EN 
         H(EN,EN) = 1.0D0 
         IF (NA .EQ. 0) GO TO 800 
C     .......... FOR I=EN-1 STEP -1 UNTIL 1 DO -- .......... 
         DO 700 II = 1, NA 
            I = EN - II 
            W = H(I,I) - P 
            R = 0.0D0 
C 
            DO 610 J = M, EN 
  610       R = R + H(I,J) * H(J,EN) 
C 
            IF (WI(I) .GE. 0.0D0) GO TO 630 
            ZZ = W 
            S = R 
            GO TO 700 
  630       M = I 
            IF (WI(I) .NE. 0.0D0) GO TO 640 
            T = W 
            IF (T .NE. 0.0D0) GO TO 635 
               TST1 = NORM 
               T = TST1 
  632          T = 0.01D0 * T 
               TST2 = NORM + T 
               IF (TST2 .GT. TST1) GO TO 632 
  635       H(I,EN) = -R / T 
            GO TO 680 
C     .......... SOLVE REAL EQUATIONS .......... 
  640       X = H(I,I+1) 
            Y = H(I+1,I) 
            Q = (WR(I) - P) * (WR(I) - P) + WI(I) * WI(I) 
            T = (X * S - ZZ * R) / Q 
            H(I,EN) = T 
            IF (DABS(X) .LE. DABS(ZZ)) GO TO 650 
            H(I+1,EN) = (-R - W * T) / X 
            GO TO 680 
  650       H(I+1,EN) = (-S - Y * T) / ZZ 
C 
C     .......... OVERFLOW CONTROL .......... 
  680       T = DABS(H(I,EN)) 
            IF (T .EQ. 0.0D0) GO TO 700 
            TST1 = T 
            TST2 = TST1 + 1.0D0/TST1 
            IF (TST2 .GT. TST1) GO TO 700 
            DO 690 J = I, EN 
               H(J,EN) = H(J,EN)/T 
  690       CONTINUE 
C 
  700    CONTINUE 
C     .......... END REAL VECTOR .......... 
         GO TO 800 
C     .......... COMPLEX VECTOR .......... 
  710    M = NA 
C     .......... LAST VECTOR COMPONENT CHOSEN IMAGINARY SO THAT 
C                EIGENVECTOR MATRIX IS TRIANGULAR .......... 
         IF (DABS(H(EN,NA)) .LE. DABS(H(NA,EN))) GO TO 720 
         H(NA,NA) = Q / H(EN,NA) 
         H(NA,EN) = -(H(EN,EN) - P) / H(EN,NA) 
         GO TO 730 
  720    CALL CDIV(0.0D0,-H(NA,EN),H(NA,NA)-P,Q,H(NA,NA),H(NA,EN)) 
  730    H(EN,NA) = 0.0D0 
         H(EN,EN) = 1.0D0 
         ENM2 = NA - 1 
         IF (ENM2 .EQ. 0) GO TO 800 
C     .......... FOR I=EN-2 STEP -1 UNTIL 1 DO -- .......... 
         DO 795 II = 1, ENM2 
            I = NA - II 
            W = H(I,I) - P 
            RA = 0.0D0 
            SA = 0.0D0 
C 
            DO 760 J = M, EN 
               RA = RA + H(I,J) * H(J,NA) 
               SA = SA + H(I,J) * H(J,EN) 
  760       CONTINUE 
C 
            IF (WI(I) .GE. 0.0D0) GO TO 770 
            ZZ = W 
            R = RA 
            S = SA 
            GO TO 795 
  770       M = I 
            IF (WI(I) .NE. 0.0D0) GO TO 780 
            CALL CDIV(-RA,-SA,W,Q,H(I,NA),H(I,EN)) 
            GO TO 790 
C     .......... SOLVE COMPLEX EQUATIONS .......... 
  780       X = H(I,I+1) 
            Y = H(I+1,I) 
            VR = (WR(I) - P) * (WR(I) - P) + WI(I) * WI(I) - Q * Q 
            VI = (WR(I) - P) * 2.0D0 * Q 
            IF (VR .NE. 0.0D0 .OR. VI .NE. 0.0D0) GO TO 784 
               TST1 = NORM * (DABS(W) + DABS(Q) + DABS(X) 
     X                      + DABS(Y) + DABS(ZZ)) 
               VR = TST1 
  783          VR = 0.01D0 * VR 
               TST2 = TST1 + VR 
               IF (TST2 .GT. TST1) GO TO 783 
  784       CALL CDIV(X*R-ZZ*RA+Q*SA,X*S-ZZ*SA-Q*RA,VR,VI, 
     X                H(I,NA),H(I,EN)) 
            IF (DABS(X) .LE. DABS(ZZ) + DABS(Q)) GO TO 785 
            H(I+1,NA) = (-RA - W * H(I,NA) + Q * H(I,EN)) / X 
            H(I+1,EN) = (-SA - W * H(I,EN) - Q * H(I,NA)) / X 
            GO TO 790 
  785       CALL CDIV(-R-Y*H(I,NA),-S-Y*H(I,EN),ZZ,Q, 
     X                H(I+1,NA),H(I+1,EN)) 
C 
C     .......... OVERFLOW CONTROL .......... 
  790       T = DMAX1(DABS(H(I,NA)), DABS(H(I,EN))) 
            IF (T .EQ. 0.0D0) GO TO 795 
            TST1 = T 
            TST2 = TST1 + 1.0D0/TST1 
            IF (TST2 .GT. TST1) GO TO 795 
            DO 792 J = I, EN 
               H(J,NA) = H(J,NA)/T 
               H(J,EN) = H(J,EN)/T 
  792       CONTINUE 
C 
  795    CONTINUE 
C     .......... END COMPLEX VECTOR .......... 
  800 CONTINUE 
C     .......... END BACK SUBSTITUTION. 
C                VECTORS OF ISOLATED ROOTS .......... 
      DO 840 I = 1, N 
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 840 
C 
         DO 820 J = I, N 
  820    Z(I,J) = H(I,J) 
C 
  840 CONTINUE 
C     .......... MULTIPLY BY TRANSFORMATION MATRIX TO GIVE 
C                VECTORS OF ORIGINAL FULL MATRIX. 
C                FOR J=N STEP -1 UNTIL LOW DO -- .......... 
      DO 880 JJ = LOW, N 
         J = N + LOW - JJ 
         M = MIN0(J,IGH) 
C 
         DO 880 I = LOW, IGH 
            ZZ = 0.0D0 
C 
            DO 860 K = LOW, M 
  860       ZZ = ZZ + Z(I,K) * H(K,J) 
C 
            Z(I,J) = ZZ 
  880 CONTINUE 
C 
      GO TO 1001 
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT 
C                CONVERGED AFTER 30*N ITERATIONS .......... 
 1000 IERR = EN 
 1001 RETURN 
      END 
      SUBROUTINE BALBAK(NM,N,LOW,IGH,SCALE,M,Z) 
C 
      INTEGER I,J,K,M,N,II,NM,IGH,LOW 
      DOUBLE PRECISION SCALE(N),Z(NM,M) 
      DOUBLE PRECISION S 
C 
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALBAK, 
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH. 
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971). 
C 
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A REAL GENERAL 
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING 
C     BALANCED MATRIX DETERMINED BY  BALANC. 
C 
C     ON INPUT 
C 
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL 
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM 
C          DIMENSION STATEMENT. 
C 
C        N IS THE ORDER OF THE MATRIX. 
C 
C        LOW AND IGH ARE INTEGERS DETERMINED BY  BALANC. 
C 
C        SCALE CONTAINS INFORMATION DETERMINING THE PERMUTATIONS 
C          AND SCALING FACTORS USED BY  BALANC. 
C 
C        M IS THE NUMBER OF COLUMNS OF Z TO BE BACK TRANSFORMED. 
C 
C        Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGEN- 
C          VECTORS TO BE BACK TRANSFORMED IN ITS FIRST M COLUMNS. 
C 
C     ON OUTPUT 
C 
C        Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE 
C          TRANSFORMED EIGENVECTORS IN ITS FIRST M COLUMNS. 
C 
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW, 
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY 
C 
C     THIS VERSION DATED AUGUST 1983. 
C 
C     ------------------------------------------------------------------ 
C 
      IF (M .EQ. 0) GO TO 200 
      IF (IGH .EQ. LOW) GO TO 120 
C 
      DO 110 I = LOW, IGH 
         S = SCALE(I) 
C     .......... LEFT HAND EIGENVECTORS ARE BACK TRANSFORMED 
C                IF THE FOREGOING STATEMENT IS REPLACED BY 
C                S=1.0D0/SCALE(I). .......... 
         DO 100 J = 1, M 
  100    Z(I,J) = Z(I,J) * S 
C 
  110 CONTINUE 
C     ......... FOR I=LOW-1 STEP -1 UNTIL 1, 
C               IGH+1 STEP 1 UNTIL N DO -- .......... 
  120 DO 140 II = 1, N 
         I = II 
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 140 
         IF (I .LT. LOW) I = LOW - II 
         K = SCALE(I) 
         IF (K .EQ. I) GO TO 140 
C 
         DO 130 J = 1, M 
            S = Z(I,J) 
            Z(I,J) = Z(K,J) 
            Z(K,J) = S 
  130    CONTINUE 
C 
  140 CONTINUE 
C 
  200 RETURN 
      END 
      SUBROUTINE CDIV(AR,AI,BR,BI,CR,CI) 
      DOUBLE PRECISION AR,AI,BR,BI,CR,CI 
C 
C     COMPLEX DIVISION, (CR,CI) = (AR,AI)/(BR,BI) 
C 
      DOUBLE PRECISION S,ARS,AIS,BRS,BIS 
      S = DABS(BR) + DABS(BI) 
      ARS = AR/S 
      AIS = AI/S 
      BRS = BR/S 
      BIS = BI/S 
      S = BRS**2 + BIS**2 
      CR = (ARS*BRS + AIS*BIS)/S 
      CI = (AIS*BRS - ARS*BIS)/S 
      RETURN 
      END 
