c     1d cartesian dynamo, explicit, for teaching purposes

c     used subroutines: arr2ps

c     files: 13 = read  'field.start'
c            14 = write 'field.end'
c            15 = write 'field.time' 

      program dynex
      implicit real*8 (a-h,o-z)                                
      implicit integer (i-n)
      parameter (nxd=201,ntd=500)
      real*4 bf(ntd,nxd)
      dimension x(nxd),sinx(nxd),cosx(nxd),a(nxd),b(nxd),
     .          anew(nxd),bnew(nxd)

c     input parameters:                                            

c     nbn  =  1 start fields calculated
c          =/ 1 start fields read from file 13
c     nx        number of gridpoints
c     dtim      increment of timestep
c     ntim      number of timesteps
c     rp        dynamo number

c     rauf      magnetic buoyancy factor
c     quen      alpha quenching factor

c     iwr  =  0 no writing of fields on file 15 (for plotting etc)
c               no butterfly image
c          =/ 0 writing of fields on file 15
c               butterfly image
c     nstwr     timestep at beginning of iwr
c     nendw     timestep at end of iwr
c     nwr       increment (iwr every nwr timesteps)

c     input parameters

      read *, nbn,nx,dtim,ntim,rp
      read *, rauf,quen
      read *, iwr,nstwr,nendw,nwr

c     output of input parameters

      print 1000, nbn,nx,dtim,ntim,rp
      print 1005, rauf,quen
      print 1010, iwr,nstwr,nendw,nwr

1000  format (/,' 1d cartesian dynamo'//,    
     .' nbn   = ',i10/,' nx    = ',i10/,
     .' dtim  = ',1pd10.3/,' ntim  = ',i10/,' rp    = ',d10.3)
1005  format (/,' rauf  = ',1pd10.3/,' quen  = ',d10.3)
1010  format (/,' iwr   = ',i10/,' nstwr = ',i10/,' nendw = ',i10/,
     .' nwr   = ',i10/)
c                                                                   
      if (nx.gt.nxd) stop 'nx>nxd'                                  

c     auxiliary variables

      nx1=nx-1                                                         
      pih=dasin(1.d0)
      dx=pih/dfloat(nx1)                                          
      dx1=1.d0/(dx*dx)
      dx2=1.d0/(2.d0*dx)

c     x grid and trigonometric functions

      do i=1,nx                                    
         x(i)=dfloat(i-1)*dx
         sinx(i)=dsin(x(i))
         cosx(i)=dcos(x(i))
      enddo

c     calculate or read start fields

      if (nbn.eq.1) then
         do i=1,nx
            b(i)=dsin(2.d0*x(i))
            a(i)=0.d0
         enddo
      else
         open (13,file='field.start',status='old',form='unformatted')
         do i=1,nx
            read (13) a(i),b(i)
         enddo
         close (13)
      end if                                                

c     for plotting etc

      if (iwr.ne.0) then
         open (15,file='field.time',status='unknown',form='formatted')
         write (15,1020) nx,nstwr,nendw,nwr
1020     format (4i8)
         do i=1,nx
            write (15,1030) x(i)
         enddo
1030     format (e15.5)
      endif

      nt=0

c     time loop
 
      do 60 k=1,ntim                                    
 
      time=dfloat(k)*dtim

c     write fields for plotting etc on unit 15
c     save fields for butterfly image

      if (iwr.eq.0) go to 30
      if (k.lt.nstwr) go to 30
      if (k.gt.nendw) go to 30
      if (mod(k,nwr).ne.0) go to 30

      write (15,1060) k,time
1060  format (i8,e15.5)
      do i=1,nx
         write (15,1070) sngl(a(i)),sngl(b(i))
      enddo
1070  format (2e15.5)

      if (nt.eq.ntd) then
         print *, 'Output image overflow at timestep / time ',k,time
      go to 30
      endif
      nt=nt+1
      do i=1,nx
         bf(nt,i)=sngl(b(i))
      enddo

30    continue

c     do euler time step
      
      do i=2,nx-1
         ip1=i+1
         im1=i-1
         da=cosx(i)*b(i)/(1.d0+quen*b(i)*b(i))
     .      +(a(ip1)-2.d0*a(i)+a(im1))*dx1
         db=rp*sinx(i)*(a(ip1)-a(im1))*dx2
     .      +(b(ip1)-2.d0*b(i)+b(im1))*dx1
     .      -rauf*dsign(1.d0,b(i))*b(i)*b(i)
         anew(i)=dtim*da+a(i)
         bnew(i)=dtim*db+b(i)
      enddo
      do i=2,nx-1
         a(i)=anew(i)
         b(i)=bnew(i)
      enddo
      a(1)=0.d0
      b(1)=0.d0
      a(nx)=(4.d0*a(nx-1)-a(nx-2))/3.d0
      b(nx)=0.d0

c     end of time loop

60    continue                                                  

      if (iwr.ne.0) close (15)

c     write end fields on unit 14

      open (14,file='field.end',status='unknown',form='unformatted')
      do i=1,nx
         write (14) a(i),b(i)
      enddo
      close (14)

      call arr2ps ('dynex.ps',bf,ntd,nt,nx,sngl(1.d0),sngl(1.d0))
c  
      stop 'normal termination'                               
      end                                                    
