c     1d cartesian dynamo, implicit, for teaching purposes
c     d. schmitt, 1 may 1995

c     used subroutines: strlp 
c                       tridag (numerical recipes)
c                       arr2ps (rick hessman)

c     files  13 = read  'field.start'
c            14 = write 'field.end'
c            15 = write 'field.time' 
c            'field.ps'

      program dynim
      implicit real*8 (a-h,o-z)                                
      implicit integer (i-n)
      parameter (nxd=201,ntd=50,nttd=500)
      character*1 chart(72)                                    
      real*4 bf(nxd,ntd),ha(2),f,fmax,fmin,fm,df,bff(nttd,nxd)
      dimension x(nxd),sinx(nxd),cosx(nxd),a(nxd),b(nxd)
     .          ,at(nxd-2),bt(nxd-2),ct(nxd-2),dta(nxd-2)
     .          ,dtb(nxd-2),xa(nxd-2),xb(nxd-2)
      data chart / '<','3',' ','2',' ','1',' ','a',' ','b',' '
     .            ,'c',56*' ','>',' ',' ','+' /                       

c     input parameters:                                            

c     nbn  =  1 start fields calculated
c          =/ 1 start fields read from file 13
c     nsw  =  1 start fields and solution symmetric to equator (quadrupol)
c          =  2                          antisymetric            (dipol)
c          =/ 1,2 whole sphere
c     nanf      only for nsw =/ 1,2 
c          =  1 start fields symetric to equator
c          =  2            antisymmetric
c          =/ 1,2           unsymmetric 
c     amp       amplitude of start field

c     nx        number of gridpoints
c     dtim      increment of timestep
c     ntim      number of timesteps
c     rp        dynamo number

c     rauf      magnetic buoyancy factor
c     quen      alpha quenching factor

c     idr  =  0 no print of fields as numbers
c          =/ 0 print of fields as numbers
c     nstdr     timestep at beginning of printing
c     nendd     timestep of end of printing
c     ndr       increment (printing every ndr timesteps)   

c     iwr  =  0 no writing of fields on file 15 (for plotting ...)
c          =/ 0 writing of fields on file 15a (for plotting ...)
c     nstwr     timestep at beginning of writing
c     nendw     timestep at end of writing
c     nwr       increment (writing every nwr timesteps)

c     iim  =  0 no butterfly image
c          =/ 0 butterfly image
c     nstim     timestep at beginning of butterfly image
c     nendi     timestep at end of butterfly image
c     nim       increment (butterfly image every nim timesteps)

c     ipl  =  0 no print plot of butterfly diagram
c          =/ 0 print plot of butterfly diagram
c     nstpl     timestep at beginning of butterfly diagram
c     nendp     timestep at end of butterfly diagram
c     npl       increment (butterfly diagram every npl timesteps)

c     input parameters

      read *, nbn,nsw,nanf,amp,nx,dtim,ntim,rp
      read *, rauf,quen
      read *, idr,nstdr,nendd,ndr,iwr,nstwr,nendw,nwr
      read *, iim,nstim,nendi,nim,ipl,nstpl,nendp,npl

c     output of input parameters

      print 1000, nbn,nsw,nanf,amp,nx,dtim,ntim,rp
      print 1005, rauf,quen
      print 1010, idr,nstdr,nendd,ndr,iwr,nstwr,nendw,nwr
      print 1015, iim,nstim,nendi,nim,ipl,nstpl,nendp,npl

1000  format (/,' 1d cartesian dynamo'//,    
     .' nbn   = ',i10/,' nsw   = ',i10/,' nanf  = ',i10/,
     .' amp   = ',1pd10.3/,' nx    = ',i10/,' dtim  = ',d10.3/,
     .' ntim  = ',i10/,' rp    = ',d10.3)
1005  format (/,' rauf  = ',1pd10.3/,' quen  = ',d10.3)
1010  format (/,                                         
     .' idr   = ',i10/,' nstdr = ',i10/,' nendd = ',i10/,
     .' ndr   = ',i10/,   
     .' iwr   = ',i10/,' nstwr = ',i10/,' nendw = ',i10/,
     .' nwr   = ',i10)
1015  format(/,
     .' iim   = ',i10/,' nstim = ',i10/,' nendi = ',i10/,
     .' nim   = ',i10/,
     .' ipl   = ',i10/,' nstpl = ',i10/,' nendp = ',i10/,
     .' npl   = ',i10/)
c                                                                   
      if (nx.gt.nxd) stop 'nx>nxd'                                  

c     auxiliary variables

      nx1=nx-1                                                         
      nx2=nx-2
      pih=dasin(1.d0)
      pi=2.d0*pih
      if (nsw.eq.1 .or. nsw.eq.2) then
         dx=pih/dfloat(nx1)
      else
         dx=pi/dfloat(nx1)                                          
      endif
      dx1=1.d0/(dx*dx)
      dx2=dtim/(2.d0*dx)
      dx3=-dtim*dx1
      dx4=1.+2.d0*dtim*dx1

c     x grid and trigonometric functions

      do i=1,nx                                    
         x(i)=dfloat(i-1)*dx
         sinx(i)=dsin(x(i))
         cosx(i)=dcos(x(i))
      enddo

c     matrices
 
      do i=1,nx2
         at(i)=dx3
         bt(i)=dx4
         ct(i)=dx3
      enddo
      atn=at(nx2)
      btn=bt(nx2)
      atx=at(nx2)-ct(nx2)/3.d0 
      btx=bt(nx2)+4.d0*ct(nx2)/3.d0

c     calculate or read start fields

      if (nbn.eq.1) then
         do i=1,nx
            a(i)=0.d0
         enddo
         if (nsw.eq.1) then
            do i=1,nx
               b(i)=amp*sinx(i)
            enddo
         else if (nsw.eq.2) then
            do i=1,nx
               b(i)=amp*dsin(2.d0*x(i))
            enddo
         else
            if (nanf.eq.1) then
               do i=1,nx
                  b(i)=amp*sinx(i)
               enddo
            else if (nanf.eq.2) then
               do i=1,nx
                  b(i)=amp*dsin(2.d0*x(i))
               enddo
            else
               do i=1,nx
                  b(i)=amp*sinx(i)*x(i)/pi
               enddo
            endif
         endif
      else
         open (13,file='field.start',status='old',form='unformatted')
         do i=1,nx
            read (13) a(i),b(i)
         enddo
         close (13)
      end if                                                

c     for plotting etc

      if (iwr.ne.0) then
         open (15,file='field.time',status='unknown',form='formatted')
         write (15,1020) nx,nstwr,nendw,nwr
1020     format (4i8)
         do i=1,nx
            write (15,1030) x(i)
         enddo
1030     format (e15.5)
      endif

      j=0
      ntt=0

c     time loop
 
      do 60 k=1,ntim                                    
 
      time=dfloat(k)*dtim

c     write fields as numbers on standard unit
      
      if (idr.eq.0) go to 20                                           
      if (k.eq.1) go to 10                                            
      if (k.lt.nstdr) go to 20                                       
      if (k.gt.nendd) go to 20
      if (mod(k,ndr).ne.0) go to 20                                 
10    print 1040, k                                               
1040  format (/,' timestep no. ',i5//,5x,'point',6x,'a',9x,    
     *        'b'/)                
      do i=1,nx
         print 1050, i,a(i),b(i)               
      enddo
1050  format (1h ,i6,4x,1pd10.2,5d10.2)                       
20    continue                                               

c     write fields for plotting etc on unit 15

      if (iwr.eq.0) go to 30
      if (k.lt.nstwr) go to 30
      if (k.gt.nendw) go to 30
      if (mod(k,nwr).ne.0) go to 30
      write (15,1060) k,time
1060  format (i8,e15.5)
      do i=1,nx
         write (15,1070) sngl(a(i)),sngl(b(i))
      enddo
1070  format (2e15.5)
30    continue

c     save fields for butterfly image

      if (iim.eq.0) go to 35
      if (k.lt.nstim) go to 35
      if (k.gt.nendi) goto 35
      if (mod(k,nim).ne.0) go to 35
      if (ntt.eq.nttd) then
         print *, 'Output image overflow at timestep / time ',k,time
         go to 35
      endif
      ntt=ntt+1
      do i=1,nx
         bff(ntt,i)=sngl(b(i))
      enddo
35    continue

c     fields for butterfly print

      if (ipl.eq.0) go to 50
      if (k.lt.nstpl) go to 50
      if (k.gt.nendp) go to 50
      if (mod(k,npl).ne.0) go to 50
      if (j.eq.0) ka=k 
      j=j+1
      k1=ntd-j+1
      do i=1,nx
         i1=nx-i+1
         bf(i1,k1)=sngl(b(i))
      enddo
      if (j.ge.ntd) then
         j=0
         ke=k
         go to 40
      end if
      go to 50

c     print butterfly diagram

40    fmax=-1000000.                                                
      fmin=1000000.                                                
      do k1=1,ntd
         do i=1,nx                                               
            f=bf(i,k1)
            fmax=amax1(f,fmax)
            fmin=amin1(f,fmin)                                
         enddo
      enddo
      fm=amax1(fmax,-fmin)                                
      df=fm/5.5                                          
      ha(1)=-fm                                         
      ha(2)=ha(1)+df                                   
      print 1080
      print 1090, ka,ke                                                
      print 1100, fmin,fmax,df                                       
1080  format (/,' b-field'/)        
1090  format (' left: equator/south-pole, right: north-pole,',          
     *        ' top: timestep = ',i5,', bottom: timestep = ',i5/)      
1100  format (' fmin = ',1pe11.3,'   fmax = ',e11.3,'   df = ',e11.3/)
      call strlp (bf,nx,ntd,ha,chart,nxd)                               
50    continue                                                    

c     do euler time step
      
c     right hand sides

      do i=1,nx2
         i1=i+1
         dta(i)=a(i1)+dtim*cosx(i1)*b(i1)/(1.d0+quen*b(i1)*b(i1))
         dtb(i)=b(i1)+dx2*rp*sinx(i1)*(a(i1+1)-a(i1-1))
     .          -dtim*rauf*dsign(1.d0,b(i1))*b(i1)*b(i1)
      enddo

c     equation for a

      if (nsw.eq.2) then
         at(nx2)=atx
         bt(nx2)=btx
      else
         at(nx2)=atn
         bt(nx2)=btn
      endif
      call tridag (at,bt,ct,dta,xa,nx2)

c     equation for b

      if (nsw.eq.1) then
         at(nx2)=atx
         bt(nx2)=btx
      else
         at(nx2)=atn
         bt(nx2)=btn
      endif
      call tridag (at,bt,ct,dtb,xb,nx2)

c     new solutions

      do i=1,nx2
         i1=i+1
         a(i1)=xa(i)
         b(i1)=xb(i)
      enddo 
      a(1)=0.d0
      b(1)=0.d0
      if (nsw.eq.1) then
         a(nx)=0.d0
         b(nx)=(4.d0*b(nx1)-b(nx2))/3.d0
      else if (nsw.eq.2) then
         a(nx)=(4.d0*a(nx1)-a(nx2))/3.d0 
         b(nx)=0.d0
      else
         a(nx)=0.d0
         b(nx)=0.d0
      endif

c     end of time loop

60    continue                                                  

      if (iwr.ne.0) close (15)

c     write end fields on unit 14

      open (14,file='field.end',status='unknown',form='unformatted')
      do i=1,nx
         write (14) a(i),b(i)
      enddo
      close (14)

c     butterfly image
 
      if (iim.ne.0) call arr2ps ('dynim.ps',bff,nttd,ntt,nx,
     .                           sngl(1.d0),sngl(1.d0))

      stop 'normal termination'                               
      end                                                    
